# -*- coding: utf-8 -*-
"""
Created on Mon Jun 12 09:46:24 2023

@author: JansenT insspired by deWitS
"""

import numpy as np
import kwant as kwant
import kwant.continuum
import sympy as sp
from matplotlib.collections import LineCollection
sp.init_printing(use_latex=True)

# plotting
from matplotlib import pyplot as plt


# =============================================================================
# Construct Hamiltonian (BHZ) for MBT
# First in basis orbital |Bi,pz,up>, i|Te,pz,up>, |Bi,pz,dw>, -i|Te,pz,dw> (4x4)
# convention pauli matrices top/bottom: tau, spin up/down: sigma, electron/hole: nu.
# =============================================================================
# Pauli matrices
sigma_0 = np.array([[1, 0], [0, 1]])
sigma_x = np.array([[0, 1], [1, 0]])
sigma_y = np.array([[0, -1j], [1j, 0]])
sigma_z = np.array([[1, 0], [0, -1]])


H_mbt = """
    + (C + D1 * k_z**2 + D2 * (k_x**2 + k_y**2)) * identity(4)
    + (M0 + B1 * k_z**2 + B2 * (k_x**2 + k_y**2)) * kron(sigma_0, sigma_z)
    + A1 * k_z * kron(sigma_z,sigma_x)
    + A2 * k_x*kron(sigma_x,sigma_x) + A2 * k_y * kron(sigma_y,sigma_x)
    + mag(site,m) * kron(sigma_z, sigma_0)
    + Vsite(site,dV) * kron(sigma_0, sigma_0)
    + Bz * kron(sigma_z, sigma_0)
  """ 
  
hamiltonian = """
       + C * identity(4) + M * kron(sigma_0, sigma_z)
       - B * (k_x**2 + k_y**2) * kron(sigma_0, sigma_z)
       - D * (k_x**2 + k_y**2) * kron(sigma_0, sigma_0)
       + A * k_x * kron(sigma_z, sigma_x)
       - A * k_y * kron(sigma_0, sigma_y)
    """
    
    
#==== system parameters ====
a = 1 #lattice paramter in x and y direction in nm
c = 1 #lattice paramter in the z direction
W = 20
L = 3
H = 7

Nx = int(L/a) #number of points in x-direction
Ny = int(W/a)
Nz = int(H/c)

#==== make estimation of relevant energy range ====
hbar = 1.054e-34
ee = 1.602e-19
me = 9.109e-31
t_energy = ((hbar**2)/(2*me*(a*10**-9)**2))/ee #energy in eV, below which model relevant

#==== Model paramaeters ====
# A1_val = -0.64*a**2
# A2_val = 3.95*a**2
# B1_val = 2.36*a**2
# B2_val = 12.87*a**2
# C_val = 0*a**2
# D1_val = -0.6*a**2
# D2_val = 8.13*a**2
# M0_val = -2*a**2

#values for Bi2Te3
# da = 10
# A1_val = 0.3/da
# A2_val = 2.87/da
# B1_val = 2.79/da**2
# B2_val = 57.38/da**2
# C_val = -0.18
# D1_val = 6.55/da**2
# D2_val = 50/da**2
# M0_val = -0.3
# m_val = 0


#Values from Yaoxin-2023
# A1_val = 0.22
# A2_val = 0.41
# B1_val = -0.1
# B2_val = -0.566
# C_val = -0.0068
# D1_val = -0.013
# D2_val = -0.196
# M0_val = 0.28
# m_val = 0.0 # postive for even layers, negetaive for odd layers

#Values from Yaoxin-2023 as initial fit, in nm
A1_val = 0.22
A2_val = 0.4429
B1_val = 0.1
B2_val = 0.3
C_val = -0.0068
D1_val = 0.0130
D2_val = 0.15
M0_val = -0.0699
#M0_val= 0.01
# m_val = 0.1 # postive for even layers, negetaive for odd layers
# Bz_val = 0.0

#Values from Zhang-2019, initially eV A, converted to eV nm, nm2
# A1_val = 2.7023/10
# A2_val = 3.1964/10
# B1_val = 11.9048/100
# B2_val = 9.4048/100
# C_val = -0.0048
# D1_val = 2.7232/100
# D2_val = 17/100
# M0_val = -0.1165
# #M0_val= 0.01


m_val = 0.2 # postive for even layers, negetaive for odd layers
Bz_val = 0.001
dV_val = 0.00 #increase of potential with every layer

def shape(site):
    (x,y,z) = site.pos

    return(0 <= y < W and 0 <= x < L and 0 <= z < H)


def lead_shape(site):
    (x,y,z) = site.pos

    return(0 <= y < W and 0 <= z < H)


def mag(site,m):
    (x,y,z) = site.pos
    if np.mod(z,2) == 1: #odd layer
        m = -m
    #if domain wall present
    # if x > 9 and (y < 18 or y > 22) and x < 21:
    #   m = -m #extra inversion
    
    # if y > 20:
    #     m = -m
    
    
    
    return m

def Vsite(site,dV):
    (x,y,z) = site.pos
    return z*dV

params = dict(A1 = A1_val, A2 = A2_val, B1 = B1_val, B2 = B2_val, C = C_val, D1 = D1_val, D2 = D2_val, M0 = M0_val, mag = mag, m = m_val, Bz = Bz_val, Vsite = Vsite, dV = dV_val)


lattice = kwant.lattice.Monatomic([[a,0,0],[0,a,0],[0,0,c]],norbs=4)
template = kwant.continuum.discretize(H_mbt,grid=lattice)

syst = kwant.Builder()
syst.fill(template,shape,(0,0,0))



lead = kwant.Builder(kwant.TranslationalSymmetry([-a,0,0]))
lead.fill(template, lead_shape, (0,0,0))

syst.attach_lead(lead)
syst.attach_lead(lead.reversed())



syst = syst.finalized()

 # # # Check that the system looks as intended.
kwant.plot(syst)


# %%
#================
#===== plot =====
#================

Nk = 181
Nb = Ny*Nz*4 #number of bands


energies = np.zeros([Nk,Nb])
eigenvectors = np.zeros([Nk,Nb,Ny,Nz,4],dtype=(np.complex_))
y_exp = np.zeros([Nk,Nb])


bands = kwant.physics.Bands(syst.leads[1], params=params)
momenta = np.linspace(-0.9*a,0.9*a,Nk)

weight = np.linspace(-1, 1,Ny)
for kk in range(Nk):
    k = momenta[kk]
    energies_k, eigenvectors_k = bands(k,return_eigenvectors=True) #returns eigenvectors as collumn vectors
    energies[kk,:] = energies_k
    eigenvectors[kk,:,:,:,:] = np.reshape(np.transpose(eigenvectors_k),[Nb,Ny,Nz,4])
    
    y_weight = np.zeros([Nb,Ny])
    for yy in range(Ny):
        y_weight[:,yy] = np.sum(np.sum(np.abs(eigenvectors[kk,:,yy,:,:])**2,2),1) #sum over 4 orbitals and height, to get y-dependence
    
    for bb in range(Nb):
        y_exp[kk,bb] = sum(y_weight[bb,:]*weight)
    
    
    
fig, axs = plt.subplots(1, 1, sharex=True, sharey=True)
for ii in range(Nb):
    #plt.plot(momenta,energies[:,ii])

    points = np.array([momenta, energies[:,ii]]).T.reshape(-1, 1, 2)
    segments = np.concatenate([points[:-1], points[1:]], axis=1)

    
    
    # Create a continuous norm to map from data points to colors
    norm = plt.Normalize(-1, 1)
    lc = LineCollection(segments, cmap='coolwarm', norm=norm)
    # Set the values used for colormapping
    lc.set_array(y_exp[:,ii])
    lc.set_linewidth(2)
    line = axs.add_collection(lc)

fig.colorbar(line, ax=axs)


ax = plt.gca()
ax.set_ylim([-0.3,0.3])
ax.set_xlim([-0.9*a,0.9*a])
plt.xlabel(r'k [nm$^{-1}$]')
plt.ylabel('Energy [eV]')

font = {'family' : 'DejaVu Sans',
        'weight' : 'normal',
        'size'   : 16}

plt.rc('font', **font)
plt.tight_layout()
#save figue
#plt.savefig('test.svg', format='svg') #svg format
#plt.savefig('BS_m=0p2_ny=3_W=40_domain_par.png',format='png', dpi = 600, transparent = True)

plt.show()

# #spin density
# spin_up_operator = np.array([[1, 0], [0, 0]])
# spinz_operator = np.kron(spin_up_operator, sigma_0)
# spin_density = kwant.operator.Density(syst, spinz_operator)
# rho_sz = sum(spin_density(psi) for psi in wave(1))
# rho_sz_reshape = np.reshape(rho_sz,[L,W,H])


# #charge density
# prob_density = kwant.operator.Density(syst,np.eye(4))
# wf = sum(prob_density(psi) for psi in wave(1))
# wf_reshape = np.reshape(wf,[L,W,H])

# #calculate expectation value based on density
# y_weight = np.mean(wf_reshape[0,:,:],1);
# #values = np.linspace(1,W,W) #along y direction
# values = np.concatenate([np.zeros([1,15]),np.linspace(1,15,15)], axis =None) #only consider left/right edge
# yexp = sum(y_weight * values)




#kwant.plot(syst)
#kwant.plotter.bands(syst.leads[1], params=params, momenta=np.linspace(-0.9, 0.9, Nk), show=False)


# %%  

def plot_wavefunction(fsyst, params,cross_plane = 1, surface_layer = H-1, Ef=-2, a=1):
    for Ef in [Ef]:
        direction = 1 #can be 0 or 1
        wave=kwant.wave_function(fsyst, energy=Ef, params=params)
        Sites=fsyst.sites
        #top surface:
        def top_surface(site1):
            return site1.pos[2]==surface_layer
       
        #cross section:
        def cross_section(site1):
            return site1.pos[0]==cross_plane
        
        
        spinz_operator = np.kron(sigma_z, sigma_0)
        spin_density = kwant.operator.Density(fsyst, spinz_operator, where=top_surface).bind(params=params)
        rho_sz = sum(spin_density(psi) for psi in wave(direction))
        
        prob_density = kwant.operator.Density(fsyst,np.eye(4),where=top_surface)
        wf = sum(prob_density(psi) for psi in wave(direction))
        
        lat2=kwant.lattice.square(norbs=1,a = a)
        sys2=kwant.Builder()
        sys2[(lat2(site.pos[0],site.pos[1]) for site in Sites if site.pos[2]==surface_layer)]=0
        sys2[lat2.neighbors()]=-1
        sys2f=sys2.finalized()
        kwant.plotter.map(sys2f, (wf), show=False)
        plt.title("Density plot of the (001) top surface, Ef = %f"%Ef)
        plt.show()
        
        kwant.plotter.map(sys2f, rho_sz, show = False, cmap='coolwarm',vmin = -0.1,vmax = 0.1)
        plt.title("Spin density plot of the (001) top surface, Ef = %f"%Ef)
        #save figure optional
        plt.savefig('Spindensity_top_m=0p2_ny=3_W=40_domainpar.svg', format = 'svg')
        plt.show()
        
        
        spin_density = kwant.operator.Density(fsyst, spinz_operator, where=cross_section).bind(params=params)
        #rho_sz = sum(spin_density(psi) for psi in wave(direction))
        
        #to only plot a single mode
        mode = 1
        rho_sz = spin_density(wave(direction)[mode])
        
        prob_density = kwant.operator.Density(fsyst,np.eye(4),where=cross_section)
        #wf = sum(prob_density(psi) for psi in wave(direction))
        
        #to only plot a single mode
        wf = prob_density(wave(direction)[mode])
        
        lat2=kwant.lattice.square(norbs=1,a = a)
        sys2=kwant.Builder()
        sys2[(lat2(site.pos[1],site.pos[2]) for site in Sites if site.pos[0]==cross_plane)]=0
        sys2[lat2.neighbors()]=-1
        sys2f=sys2.finalized()
        kwant.plotter.map(sys2f, (wf), show=False)
        plt.title("Density plot of the cross section of the (100) plane, Ef = %f"%Ef)
        plt.show()
        
        kwant.plotter.map(sys2f, rho_sz, show = False, cmap='coolwarm',vmin = -0.1, vmax = 0.1)
        plt.title("Spin density plot of the cross section of the (100) plane, Ef = %f"%Ef)
        #save figure optional
        plt.savefig('SpinDensity_m=0p2_ny=11_W=20.svg', format = 'svg')
        plt.show()
        
        
        
    return np.log(wf)



# %% plot wavefunction part

Ef = energies[11,int(Nb/2)]
plot_wavefunction(syst,params, surface_layer = 0, Ef = -0.07)

# %%

# Ne = 64 #number of points to compute conductance
# Elow = -0.2  #lower limit
# Ehig = 0.2 #upper limit

# energies = np.linspace(Elow,Ehig,Ne)
#Ef = -0.13
#trans =kwant.smatrix(syst, Ef, params=params)
#print(str(trans.transmission(1,0)))


# # Compute conductance
# data = []
# for energy in energies:
#     #params.update({'mu_left':mu_left_val+0.5*energy,'mu_right':mu_right_val-0.5*energy})
#     #smatrix = kwant.smatrix(syst, 0.004 ,params = params)
#     smatrix = kwant.smatrix(syst, energy, params=params)
#     # Conductance is N - R_ee + R_he
#     data.append(smatrix.transmission(1, 0))
    

# plt.figure()
# plt.plot(energies, data,[energies[0],energies[Ne-1]],[1,1])

# ax = plt.gca()
# #ax.set_ylim([-0.3,0.3])
# #ax.set_xlim([-0.9*a,0.9*a])
# plt.xlabel('Energy [eV]')
# plt.ylabel('Conductance [G$_0$]')

# font = {'family' : 'DejaVu Sans',
#         'weight' : 'normal',
#         'size'   : 16}

# plt.rc('font', **font)

# plt.show()
#plt.savefig(results_dir + 'Conductance.png', format='png', dpi = 600, transparent = True)